Unit AVLTree;

Interface

Uses ComCtrls;

Type
      PPavlUzel = ^PavlUzel;
      PavlUzel = ^TavlUzel;
      TavlUzel = Record
                 Synove : Array [0..2] Of PavlUzel;
                 Value    : Integer;
                 end;

Function  avlInsert(Var Root:PavlUzel; Value:Integer):Boolean;
Function  avlDelete(Var Root:PavlUzel; Value:Integer):Boolean;
Function  avlFind(Root:PavlUzel; Value:Integer):PavlUzel;
Procedure avlDestroy(Root:PavlUzel);
Procedure avlVykresli(Root:PavlUzel; TV:TTreeView);
Function avlMaxDepth(Root:PavlUzel):Integer;

Implementation

Uses SysUtils;

 Const
      RTYPE_RIGHT = -1;
      RTYPE_LEFT  =  1;
      LEFT_SON    =  0;
      RIGHT_SON   =  2;
      BALANCE     =  1;

 Function _avlSucc(Node:PavlUzel; Parent:PPavlUzel = Nil):PavlUzel;
 begin
 If Assigned(Parent) Then
    Parent^:=Node;
 Result:=Node.Synove[RIGHT_SON];
 While Assigned(Result.Synove[LEFT_SON]) Do
       begin
       If Assigned(Parent) Then
          Parent^:=Result;
       Result:=Result.Synove[LEFT_SON];
       end;
 end;

 Function _avlPred(Node:PavlUzel; Parent:PPavlUzel = Nil):PavlUzel;
 begin
 If Assigned(Parent) Then
    Parent^:=Node;
 Result:=Node.Synove[LEFT_SON];
 While Assigned(Result.Synove[RIGHT_SON]) Do
       begin
       If Assigned(Parent) Then
          Parent^:=Result;
       Result:=Result.Synove[RIGHT_SON];
       end;
 end;

 Function __opak(rType:Integer):Integer;
 begin
 Result:= -rType;
 end;

 Procedure _avlRotate(Var Root:PavlUzel; rType:Integer);
 Var Pivot : PavlUzel;
 begin
 Pivot:=Root.Synove[rType+1];
 Root.Synove[rType+1]:=Pivot.Synove[__opak(rType)+1];
 Pivot.Synove[__opak(rType)+1]:=Root;
 Root:=Pivot;
 end;

 Function _Rebalance(Var Root:PavlUzel; R:Integer):Boolean;
 Var Pom  : PavlUzel;
     _Pom : Integer;
 begin
 Result:=True;
 If Integer(Root.Synove[BALANCE]) = -R*2 Then
    begin
    If Integer(Root.Synove[-R+1].Synove[BALANCE]) = 0 Then
       begin
       _avlRotate(Root,-R);
       Integer(Root.Synove[BALANCE]):=R;
       Integer(Root.Synove[R+1].Synove[BALANCE]):=-R;
       Result:=False;
       end
    Else If Integer(Root.Synove[-R+1].Synove[BALANCE]) = -R Then
         begin
         _avlRotate(Root,-R);
         Integer(Root.Synove[BALANCE]):=0;
         Integer(Root.Synove[R+1].Synove[BALANCE]):=0;
         end
    Else If Integer(Root.Synove[-R+1].Synove[BALANCE]) = R Then
            begin
            _Pom:=Integer(Root.Synove[-R+1].Synove[R+1].Synove[BALANCE]);
            _avlRotate(Root.Synove[-R+1],R);
            _avlRotate(Root,-R);
            Integer(Root.Synove[BALANCE]):=0;
            Integer(Root.Synove[LEFT_SON].Synove[BALANCE]):=0;
            Integer(Root.Synove[RIGHT_SON].Synove[BALANCE]):=0;
            If _Pom <> 0 Then
                Integer(Root.Synove[-_Pom+1].Synove[BALANCE]):=-_Pom;
            end;
    end;
 end;

Function avlInsert(Var Root:PavlUzel; Value:Integer):Boolean;
Var R   : Integer;
    Pom : Integer;
begin
If Not Assigned(Root) Then
   begin
   New(Root);
   Root.Value:=Value;
   Root.Synove[LEFT_SON]:=Nil;
   Root.Synove[RIGHT_SON]:=Nil;
   Integer(Root.Synove[BALANCE]):=0;
   Result:=True;
   end
Else begin
     If Root.Value = Value Then
        Result:=False
     Else begin
          R:=Value - Root.Value;
          R:=R Div Abs(R);
          If R < 0 Then
             Result:=avlInsert(Root.Synove[LEFT_SON],Value)
          Else Result:=avlInsert(Root.Synove[RIGHT_SON],Value);
          If Result Then
             begin
             Inc(Integer(Root.Synove[BALANCE]),R);
             If Integer(Root.Synove[BALANCE]) = 0 Then
                Result:=False
             Else begin
                  _Rebalance(Root,-R);
                  Result:= Integer(Root.Synove[BALANCE]) <> 0;
                  end;
             end;
          end;
     end;
end;

Function avlDelete(Var Root:PavlUzel; Value:Integer):Boolean;
Var Pom    : PavlUzel;
    X      : Integer;
    R      : Integer;
    _Pom    : Integer;
begin
If Not Assigned(Root) Then
   Result:=False
Else begin
     If Root.Value = Value Then
        begin
        Result:=True;
        If (Root.Synove[LEFT_SON] = Nil) And (Root.Synove[RIGHT_SON] = Nil) Then
           begin
           Dispose(Root);
           Root:=Nil;
           end
        Else If (Root.Synove[LEFT_SON] = Nil) Then
                begin
                Pom:=Root;
                Root:=Root.Synove[RIGHT_SON];
                Dispose(Pom);
                end
        Else If (Root.Synove[RIGHT_SON] = Nil) Then
                begin
                Pom:=Root;
                Root:=Root.Synove[LEFT_SON];
                Dispose(Pom);
                end
        Else begin
             Pom:=_avlPred(Root);
             X:=Pom.Value;
             Pom.Value:=Root.Value;
             Root.Value:=X;
             Result:=avlDelete(Root.Synove[LEFT_SON],Value);
             If Result Then
                begin
                R:=-1;
                Dec(Integer(Root.Synove[BALANCE]),R);
                If (Integer(Root.Synove[BALANCE]) = 1)  Or
                   (Integer(Root.Synove[BALANCE]) = -1) Then
                   Result:=False
                Else Result:=_Rebalance(Root,R);
                end;
             end;
        end
     Else begin
          R:=Value - Root.Value;
          R:= R Div Abs(R);
          If R < 0 Then
             Result:=avlDelete(Root.Synove[LEFT_SON],Value)
          Else Result:=avlDelete(Root.Synove[RIGHT_SON],Value);
          If Result Then
             begin
             Dec(Integer(Root.Synove[BALANCE]),R);
             If (Integer(Root.Synove[BALANCE]) = 1)  Or
                (Integer(Root.Synove[BALANCE]) = -1) Then
                Result:=False
             Else Result:=_Rebalance(Root,R);
             end;
          end;
     end;
end;

Function avlFind(Root:PavlUzel; Value:Integer):PavlUzel;
begin
If Not Assigned(Root) Then
   Result:=Nil
Else begin
     If Root.Value = Value Then
        Result:=Root
     Else If Root.Value > Value Then
             Result:=avlFind(Root.Synove[LEFT_SON],Value)
     Else Result:=avlFind(Root.Synove[RIGHT_SON],Value);
     end;
end;

Procedure avlDestroy(Root:PavlUzel);
begin
If Assigned(Root) Then
   begin
   avlDestroy(Root.Synove[LEFT_SON]);
   avlDestroy(Root.Synove[RIGHT_SON]);
   Dispose(Root);
   end;
end;

Procedure avlVykresli(Root:PavlUzel; TV:TTreeView);

 Procedure _Vykresli(Root:PavlUzel; Node:TTreeNode);
 Var N : TTreeNode;
 begin
 If Assigned(Root) Then
    begin
    If Abs(Integer(Root.Synove[BALANCE])) > 1 Then
       Halt;
    N:=TV.Items.AddChild(Node,IntToStr(Root.Value)+' / '+IntToStr(Integer(Root.Synove[BALANCE])));
    _Vykresli(Root.Synove[LEFT_SON],N);
    _Vykresli(Root.Synove[RIGHT_SON],N);
    end;
 end;


Var N : TTreeNode;
begin
TV.Items.Clear;
If Assigned(Root) Then
   begin
    If Abs(Integer(Root.Synove[BALANCE])) > 1 Then
       Halt;
   N:=TV.Items.Add(Nil,IntToStr(Root.Value)+' / '+IntToStr(Integer(Root.Synove[BALANCE]))+' - AVL Root');
   _Vykresli(Root.Synove[LEFT_SON],N);
   _Vykresli(Root.Synove[RIGHT_SON],N);
   TV.AutoExpand:=True;
   end;
end;

Function avlMaxDepth(Root:PavlUzel):Integer;

Var Max : Integer;

 Procedure _Max(Root:PavlUzel; Akt:Integer);
 begin
 If Assigned(Root) Then
    begin
    If Akt > Max Then
       Max:=Akt;
    _Max(Root.Synove[LEFT_SON],Akt+1);
    _Max(Root.Synove[RIGHT_SON],Akt+1);
    end;
 end;

begin
Max:=0;
_Max(Root,0);
Result:=Max;
end;


End.
